Imports System.Reflection
Imports System.Collections.Generic

<AttributeUsage(AttributeTargets.Field)> _
Public Class DisposableAttribute : Inherits Attribute

    Private Shared _MyType As Type = GetType(DisposableAttribute)
    Private Shared _AttributedFields As New Dictionary(Of Type, FieldInfo())

    Public Shared Sub DisposeFields(ByVal itm As Object)
        DisposeFields(itm.GetType, itm)
    End Sub

    ''' <exception cref="System.InvalidCastException">
    '''  Wenn eines der als Disposable attributierten Felder von tp gar nicht 
    '''   IDisposable implementiert
    ''' </exception>
    ''' <param name="tp">Der Typ, dessen Felder berprft werden</param>
    ''' <param name="itm">Eine Instanz dieses Typs</param>
    Public Shared Sub DisposeFields(ByVal tp As Type, ByVal itm As Object)
        Dim fieldInfos As FieldInfo() = Nothing

        If Not _AttributedFields.TryGetValue(tp, fieldInfos) Then
            ' Wenn der Typ tp noch unbekannt ist, einen Eintrag tp / fieldInfos 
            '  von allen FieldInfos des Typs anlegen und die mit <Disposable> 
            '  attributierten aufsammeln
            Dim flds As FieldInfo() = itm.GetType.GetFields( _
                BindingFlags.Instance Or BindingFlags.GetField Or _
                BindingFlags.NonPublic Or BindingFlags.Public)
            Dim i As Integer
            For i = 0 To flds.Length - 1
                If Not flds(i).IsDefined(DisposableAttribute._MyType, False) _
                    Then Exit For
            Next
            For ii As Integer = i + 1 To flds.Length - 1
                If flds(ii).IsDefined(DisposableAttribute._MyType, False) Then
                    flds(i) = flds(ii)
                    i += 1
                End If
            Next
            ReDim fieldInfos(i - 1)
            Array.Copy(flds, fieldInfos, i)
            _AttributedFields.Add(tp, fieldInfos)
        End If

        ' Alle Werte der fieldInfos disposen
        For Each fld As FieldInfo In fieldInfos
            Dim fieldValue As Object = fld.GetValue(itm)
            If fieldValue IsNot Nothing Then
                Debug.WriteLine("disposing " & fld.Name)
                DirectCast(fieldValue, IDisposable).Dispose()
            End If
        Next
    End Sub

End Class
